//===-- temu-c/Cpu.h - Micro-processor functions ----------------*- C++ -*-===//
//
// T-EMU: The Terma Emulator
// (c) Terma 2015
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_SUPPORT_CPU
#define TEMU_SUPPORT_CPU

/*!
  \file Cpu.h

  \brief Wrappers functions for the CPU interfaces.

  The functions declared in this file exists for convenience
  purposes. They will in general look up the relevant interface by
  name via a double map lookup.

  The functions here provide an easy way invoke functions in the
  CpuIface without having to query for the interface and then calling
  the function.

  If the emulator is integrated in a simulator, the recommended
  approach is to first construct the system, and then query relevant
  interfaces, caching them as needed. Note that system configurations
  should not normally change after the construction phase, so any
  cached object-interface pairs will be stable.

  \warning A general rule is that the functions defined here calls
  abort in-case the interface cannot be found for the relevant object.
*/

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/*!
  \brief Get the clock frequency for the CPU

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object
  \result The configured clock frequency in Hz.
 */
uint64_t temu_cpuGetFreq(void *Cpu);

/*!
  \brief Set the program counter

  The program counter will be set to the supplied value.

  \note For targets with delay slots (SPARC, MIPS etc), the function
  will also set the next PC to PC + sizeof(instruction).

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object
  \param Pc The program counter.
 */
void temu_cpuSetPc(void *Cpu, uint64_t Pc);

/*!
  \brief Get the program counter

  The program counter will be returned.

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object
  \result The value of the program counter register
 */
uint64_t temu_cpuGetPc(void *Cpu);

/*!
  \brief Reset the processor.

  Resetting the CPU will result in a reset cascade where all connected
  devices are also reset.

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object

  \param ResetType The type of reset, by convention 0 means cold
                   reset, and 1 indicates a warm reset.
 */
void temu_cpuReset(void *Cpu, int ResetType);

/*!
  \brief Run the processor for a number of cycles

  The function runs the processor for a number of cycles. If you wish
  to run the processor with another time unit, you can compute the
  cycles from the clock frequency of the emulated processor.

  In case the processor halts or enters idle mode and there are no
  pending events the function will return early.

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object
  \param Cycles The number of cycles to run the processor for.
  \result The number of executed cycles.
 */
uint64_t temu_cpuRun(void *Cpu, uint64_t Cycles);

/*!
  \brief Run the processor for a number of steps

  This function is different from temu_cpuRun, which runs for a
  time. The steps here indicates instructions executed (including
  trapping instructions). This can be contrasted to the run function
  which may advance the cycle counter by more than one for an
  instruction (depending on the timing models).

  The function may return early in case the processor halts its
  execution or has entered idle mode and there are no events pending.

  \warning In case the Cpu does not implement the CpuIface the program
  will abort. Do not use this function in performance critical code!!!

  \param Cpu The CPU object
  \param Steps The number of steps to run the processor for.
  \result The number of executed steps.
 */
uint64_t temu_cpuStep(void *Cpu, uint64_t Steps);


uint64_t temu_cpuGetReg(void *Cpu, unsigned Reg);
void temu_cpuSetReg(void *Cpu, unsigned Reg, uint64_t Value);

float temu_cpuGetFpr32(void *Cpu, unsigned Reg);
uint32_t temu_cpuGetFpr32Bits(void *Cpu, unsigned Reg);
void temu_cpuSetFpr32(void *Cpu, unsigned Reg, float Value);
void temu_cpuSetFpr32Bits(void *Cpu, unsigned Reg, uint32_t Value);

double temu_cpuGetFpr64(void *Cpu, unsigned Reg);
uint64_t temu_cpuGetFpr64Bits(void *Cpu, unsigned Reg);
void temu_cpuSetFpr64(void *Cpu, unsigned Reg, double Value);
void temu_cpuSetFpr64Bits(void *Cpu, unsigned Reg, uint64_t Value);

void temu_cpuEnableTraps(void *Cpu);
void temu_cpuDisableTraps(void *Cpu);

/* Sparc specific functions */

typedef void (*temu_SparcAsrHandler)(void *Cpu, uint32_t Instr);
//typedef void (*temu_SparcAsiHandler)(void *Cpu, temu_MemTransaction *MT);

int temu_sparcGetWindowCount(void *Cpu);
uint32_t temu_sparcGetWindowedReg(void *Cpu, int Window, unsigned Reg);
void temu_sparcSetWindowedReg(void *Cpu, int Window, unsigned Reg, uint32_t Value);

void temu_sparcSetY(void *Cpu, uint64_t Value);
uint64_t temu_sparcGetY(void *Cpu);

void temu_sparcSetAsr(void *Cpu, unsigned Reg, uint64_t Value);
uint64_t temu_sparcGetAsr(void *Cpu, unsigned Reg);
void temu_sparcSetAsrWriter(void *Cpu, unsigned Asr, temu_SparcAsrHandler Handler);
void temu_sparcSetAsrReader(void *Cpu, unsigned Asr, temu_SparcAsrHandler Handler);

void temu_sparcSetPsr(void *Cpu, uint32_t Value);
uint32_t temu_sparcGetPsr(void *Cpu);

void temu_sparcSetTbr(void *Cpu, uint32_t Value);
uint32_t temu_sparcGetTbr(void *Cpu);

void temu_sparcSetWim(void *Cpu, uint32_t Value);
uint32_t temu_sparcGetWim(void *Cpu);

void temu_sparcSetNPc(void *Cpu, uint32_t Value);
uint32_t temu_sparcGetNPc(void *Cpu);

// Machine functions


/*!
  \brief Reset the Machine

  Resetting the Machine will result in a reset cascade where all connected
  devices and processors are also reset.

  \warning In case the Machine does not implement the MachineIface the
  program will abort. Do not use this function in performance critical
  code!!!

  \param Machine The Machine object

  \param ResetType The type of reset, by convention 0 means cold
                   reset, and 1 indicates a warm reset.
 */
void temu_machineReset(void *Machine, int ResetType);


/*!
  \brief Run the machine for a number of nanoseconds

  The function runs the machine and for a number of nanoseconds. This
  will run all the CPUs in the system.

  In case a processor halts or enters idle mode, the function returns
  early.

  \warning In case the machine does not implement the MachineIface the
  program will abort. Do not use this function in performance critical
  code!!!

  \param Machine The machine object
  \param NanoSecs The number of nanosecs to run each processor for.
  \result The number of executed nanoseconds.
 */
uint64_t temu_machineRun(void *Machine, uint64_t NanoSecs);


#ifdef __cplusplus
}
#endif

#endif /* ! TEMU_SUPPORT_CPU */
